@@ -28,7 +28,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>		/* For uid_t, gid_t */
+#include <sys/types.h>          /* For uid_t, gid_t */
 
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
@@ -36,12 +36,9 @@
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
 
-#include <libgnomeui/gnome-ui-init.h>
-#include <libgnomevfs/gnome-vfs-async-ops.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gio/gio.h>
 
-#include "file-transfer-dialog.h"
+#include "copy-theme-dialog.h"
 
 #include "gs-theme-manager.h"
 #include "gs-job.h"
@@ -49,6 +46,9 @@
 
 #define GLADE_XML_FILE "gnome-screensaver-preferences.glade"
 
+#define GNOME_LOCKDOWN_DIR  "/desktop/gnome/lockdown"
+#define KEY_LOCK_DISABLE    GNOME_LOCKDOWN_DIR "/disable_lock_screen"
+
 #define KEY_DIR             "/apps/gnome-screensaver"
 #define KEY_LOCK            KEY_DIR "/lock_enabled"
 #define KEY_IDLE_ACTIVATION_ENABLED         KEY_DIR "/idle_activation_enabled"
@@ -325,6 +325,20 @@ config_get_lock (gboolean *is_writable)
         return lock;
 }
 
+static gboolean
+config_get_lock_disabled ()
+{
+        GConfClient *client;
+        gboolean     lock;
+
+        client = gconf_client_get_default ();
+
+        lock = gconf_client_get_bool (client, KEY_LOCK_DISABLE, NULL);
+
+        g_object_unref (client);
+        return lock;
+}
+
 static void
 config_set_lock (gboolean lock)
 {
@@ -408,53 +422,53 @@ preview_set_theme (GtkWidget  *widget,
 static void
 help_display (void)
 {
-	GError     *error = NULL;
-	char       *command;
-	const char *lang;
-	char       *uri = NULL;
-	GdkScreen  *gscreen;
-	int         i;
+        GError     *error = NULL;
+        char       *command;
+        const char *lang;
+        char       *uri = NULL;
+        GdkScreen  *gscreen;
+        int         i;
 
-	const char * const * langs = g_get_language_names ();
+        const char * const * langs = g_get_language_names ();
 
-	for (i = 0; langs[i] != NULL; i++) {
-		lang = langs[i];
-		if (strchr (lang, '.')) {
-			continue;
-		}
+        for (i = 0; langs[i] != NULL; i++) {
+                lang = langs[i];
+                if (strchr (lang, '.')) {
+                        continue;
+                }
 
-		uri = g_build_filename (DATADIR,
+                uri = g_build_filename (DATADIR,
                                         "/gnome/help/user-guide/",
-					lang,
+                                        lang,
                                         "/user-guide.xml",
-					NULL);
+                                        NULL);
 
-		if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
+                if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
                     break;
-		}
-	}
+                }
+        }
 
-	command = g_strconcat ("gnome-open ghelp://",
+        command = g_strconcat ("gnome-open ghelp://",
                                uri,
                                "?prefs-screensaver",
                                NULL);
-	gscreen = gdk_screen_get_default ();
-	gdk_spawn_command_line_on_screen (gscreen, command, &error);
+        gscreen = gdk_screen_get_default ();
+        gdk_spawn_command_line_on_screen (gscreen, command, &error);
 
-	if (error != NULL) {
-		GtkWidget *d;
+        if (error != NULL) {
+                GtkWidget *d;
 
-		d = gtk_message_dialog_new (NULL,
+                d = gtk_message_dialog_new (NULL,
                                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                             GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
                                             "%s", error->message);
-		gtk_dialog_run (GTK_DIALOG (d));
-		gtk_widget_destroy (d);
-		g_error_free (error);
-	}
+                gtk_dialog_run (GTK_DIALOG (d));
+                gtk_widget_destroy (d);
+                g_error_free (error);
+        }
 
-	g_free (command);
-	g_free (uri);
+        g_free (command);
+        g_free (uri);
 }
 
 static void
@@ -811,123 +825,30 @@ reload_themes (void)
 }
 
 static void
-transfer_done_cb (GtkWidget *dialog,
-                  char      *path)
+theme_copy_complete_cb (GtkWidget *dialog, gpointer user_data)
 {
-
-        g_free (path);
-
-        gtk_widget_destroy (dialog);
-
         reload_themes ();
-}
-
-static void
-transfer_cancel_cb (GtkWidget *dialog,
-                    char      *path)
-{
-        gnome_vfs_unlink (path);
-        g_free (path);
-
         gtk_widget_destroy (dialog);
 }
 
 static void
-theme_installer_run (GtkWidget *parent,
-                     char      *filename)
+theme_installer_run (GtkWidget *prefs_dialog, GList *files)
 {
-        GtkWidget    *dialog;
-        GnomeVFSURI  *src_uri;
-        GList        *src;
-        GList        *target;
-        char         *user_dir;
-        char         *short_name;
-        char         *base_name;
-        char         *target_path;
-        gboolean      is_desktop;
+        GtkWidget *copy_dialog;
 
-        src_uri = gnome_vfs_uri_new (filename);
-        src = g_list_append (NULL, src_uri);
-        target = NULL;
-        target_path = NULL;
+        copy_dialog = copy_theme_dialog_new (files);
+        g_list_foreach (files, (GFunc) (g_object_unref), NULL);
+        g_list_free (files);
 
-        user_dir = g_build_filename (g_get_user_data_dir (), "applications", "screensavers", NULL);
-        g_mkdir_with_parents (user_dir, S_IRWXU);
+        gtk_window_set_transient_for (GTK_WINDOW (copy_dialog),
+                                        GTK_WINDOW (prefs_dialog));
+        gtk_window_set_icon_name (GTK_WINDOW (copy_dialog),
+                                        "preferences-desktop-screensaver");
 
-        short_name = gnome_vfs_uri_extract_short_name (src_uri);
-        base_name = NULL;
+        g_signal_connect (copy_dialog, "complete",
+                          G_CALLBACK (theme_copy_complete_cb), NULL);
 
-        is_desktop = FALSE;
-        if (short_name != NULL
-            && g_str_has_suffix (short_name, ".desktop")) {
-                /* FIXME: validate key file? */
-                is_desktop = TRUE;
-                base_name = g_strndup (short_name, strlen (short_name) - 8);
-        }
-
-        while (TRUE) {
-                char *tmp;
-                gboolean exists;
-
-                if (! is_desktop) {
-                        dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-                                                         GTK_DIALOG_MODAL,
-                                                         GTK_MESSAGE_ERROR,
-                                                         GTK_BUTTONS_OK,
-                                                         _("Invalid screensaver theme"));
-                        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                                  "%s",
-                                                                  _("This file does not appear to be a valid screensaver theme."));
-                        gtk_window_set_title (GTK_WINDOW (dialog), "");
-                        gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
-
-                        gtk_dialog_run (GTK_DIALOG (dialog));
-                        gtk_widget_destroy (dialog);
-                        g_free (target_path);
-                        goto out;
-                }
-
-                g_free (target_path);
-                target_path = g_build_filename (user_dir, short_name, NULL);
-
-                /* FIXME: racy, oh well */
-                exists = gnome_vfs_uri_exists (gnome_vfs_uri_new (target_path));
-                if (! exists) {
-                        target = g_list_append (NULL, gnome_vfs_uri_new (target_path));
-                        break;
-                }
-
-                /* try another name */
-                tmp = g_strdup_printf ("%s-%u.desktop",  base_name, g_random_int ());
-                g_free (short_name);
-                short_name = tmp;
-        }
-
-        dialog = file_transfer_dialog_new ();
-        gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
-        gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
-
-        file_transfer_dialog_wrap_async_xfer (FILE_TRANSFER_DIALOG (dialog),
-                                              src, target,
-                                              GNOME_VFS_XFER_RECURSIVE,
-                                              GNOME_VFS_XFER_ERROR_MODE_QUERY,
-                                              GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
-                                              GNOME_VFS_PRIORITY_DEFAULT);
-
-        g_signal_connect (dialog, "cancel",
-                          G_CALLBACK (transfer_cancel_cb), target_path);
-        g_signal_connect (dialog, "done",
-                          G_CALLBACK (transfer_done_cb), target_path);
-
-        gtk_widget_show (dialog);
-
- out:
-        gnome_vfs_uri_list_unref (src);
-        gnome_vfs_uri_list_unref (target);
-
-        g_free (user_dir);
-        g_free (base_name);
-        g_free (short_name);
+        copy_theme_dialog_begin (COPY_THEME_DIALOG (copy_dialog));
 }
 
 /* Callback issued during drag movements */
@@ -946,12 +867,69 @@ drag_motion_cb (GtkWidget      *widget,
 static void
 drag_leave_cb (GtkWidget      *widget,
                GdkDragContext *context,
-	       guint           time,
+               guint           time,
                gpointer        data)
 {
         gtk_widget_queue_draw (widget);
 }
 
+/* GIO has no version of gnome_vfs_uri_list_parse(), so copy from GnomeVFS
+ * and re-work to create GFiles.
+**/
+static GList *
+uri_list_parse (const gchar *uri_list)
+{
+        const gchar *p, *q;
+        gchar *retval;
+        GFile *file;
+        GList *result = NULL;
+
+        g_return_val_if_fail (uri_list != NULL, NULL);
+
+        p = uri_list;
+
+        /* We don't actually try to validate the URI according to RFC
+         * 2396, or even check for allowed characters - we just ignore
+         * comments and trim whitespace off the ends.  We also
+         * allow LF delimination as well as the specified CRLF.
+         */
+        while (p != NULL) {
+                if (*p != '#') {
+                        while (g_ascii_isspace (*p))
+                                p++;
+
+                        q = p;
+                        while ((*q != '\0')
+                               && (*q != '\n')
+                               && (*q != '\r'))
+                                q++;
+
+                        if (q > p) {
+                                q--;
+                                while (q > p
+                                       && g_ascii_isspace (*q))
+                                        q--;
+
+                                retval = g_malloc (q - p + 2);
+                                strncpy (retval, p, q - p + 1);
+                                retval[q - p + 1] = '\0';
+
+                                file = g_file_new_for_uri (retval);
+
+                                g_free (retval);
+
+                                if (file != NULL)
+                                        result = g_list_prepend (result, file);
+                        }
+                }
+                p = strchr (p, '\n');
+                if (p != NULL)
+                        p++;
+        }
+
+        return g_list_reverse (result);
+}
+
 /* Callback issued on actual drops. Attempts to load the file dropped. */
 static void
 drag_data_received_cb (GtkWidget        *widget,
@@ -963,32 +941,18 @@ drag_data_received_cb (GtkWidget        *widget,
                        guint             time,
                        gpointer          data)
 {
-        GtkWidget *dialog;
-        GList     *uris;
-        char      *filename = NULL;
+        GList     *files;
 
         if (!(info == TARGET_URI_LIST || info == TARGET_NS_URL))
                 return;
 
-        uris = gnome_vfs_uri_list_parse ((char *) selection_data->data);
-        if (uris != NULL && uris->data != NULL) {
-                GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
+        files = uri_list_parse ((char *) selection_data->data);
+        if (files != NULL) {
+                GtkWidget *prefs_dialog;
 
-                if (gnome_vfs_uri_is_local (uri)) {
-                        filename = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri),
-                                                              G_DIR_SEPARATOR_S);
-                } else {
-                        filename = gnome_vfs_unescape_string (gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE),
-                                                              G_DIR_SEPARATOR_S);
-                }
-
-                gnome_vfs_uri_list_unref (uris);
+                prefs_dialog = glade_xml_get_widget (xml, "prefs_dialog");
+                theme_installer_run (prefs_dialog, files);
         }
-
-        dialog = glade_xml_get_widget (xml, "prefs_dialog");
-        theme_installer_run (dialog, filename);
-
-        g_free (filename);
 }
 
 /* Adapted from totem_time_to_string_text */
@@ -1073,11 +1037,24 @@ invalid_type_warning (const char *type)
                    type);
 }
 
+static void
+ui_disable_lock (gboolean disable)
+{
+        GtkWidget *widget;
+
+        widget = glade_xml_get_widget (xml, "lock_checkbox");
+        gtk_widget_set_sensitive (widget, !disable);
+        if (disable) {
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+        }
+}
+
 static void
 ui_set_lock (gboolean enabled)
 {
         GtkWidget *widget;
         gboolean   active;
+        gboolean   lock_disabled;
 
         widget = glade_xml_get_widget (xml, "lock_checkbox");
 
@@ -1085,6 +1062,8 @@ ui_set_lock (gboolean enabled)
         if (active != enabled) {
                 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), enabled);
         }
+        lock_disabled = config_get_lock_disabled ();
+        ui_disable_lock (lock_disabled);
 }
 
 static void
@@ -1093,6 +1072,7 @@ ui_set_enabled (gboolean enabled)
         GtkWidget *widget;
         gboolean   active;
         gboolean   is_writable;
+        gboolean   lock_disabled;
 
         widget = glade_xml_get_widget (xml, "enable_checkbox");
         active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
@@ -1105,6 +1085,8 @@ ui_set_enabled (gboolean enabled)
         if (is_writable) {
                 gtk_widget_set_sensitive (widget, enabled);
         }
+        lock_disabled = config_get_lock_disabled ();
+        ui_disable_lock(lock_disabled);
 }
 
 static void
@@ -1127,7 +1109,7 @@ key_changed_cb (GConfClient *client,
 
         key = gconf_entry_get_key (entry);
 
-        if (! g_str_has_prefix (key, KEY_DIR)) {
+        if (! g_str_has_prefix (key, KEY_DIR) && ! g_str_has_prefix (key, GNOME_LOCKDOWN_DIR)) {
                 return;
         }
 
@@ -1153,6 +1135,16 @@ key_changed_cb (GConfClient *client,
                 } else {
                         invalid_type_warning (key);
                 }
+        } else if (strcmp (key, KEY_LOCK_DISABLE) == 0) {
+                if (value->type == GCONF_VALUE_BOOL) {
+                        gboolean disabled;
+
+                        disabled = gconf_value_get_bool (value);
+
+                        ui_disable_lock (disabled);
+                } else {
+                        invalid_type_warning (key);
+                }
         } else if (strcmp (key, KEY_THEMES) == 0) {
                 if (value->type == GCONF_VALUE_LIST) {
                         GtkWidget *treeview;
@@ -1514,7 +1506,7 @@ init_capplet (void)
 
         setup_list_size_constraint (list_scroller, treeview);
         gtk_widget_set_size_request (preview, 480, 300);
-        gtk_window_set_icon_name (GTK_WINDOW (dialog), "screensaver");
+        gtk_window_set_icon_name (GTK_WINDOW (dialog), "preferences-desktop-screensaver");
         gtk_window_set_icon_name (GTK_WINDOW (fullscreen_preview_window), "screensaver");
 
         gtk_drag_dest_set (dialog, GTK_DEST_DEFAULT_ALL,
@@ -1549,12 +1541,19 @@ init_capplet (void)
                 }
         }
 
-	gconf_client_add_dir (client, KEY_DIR,
-			      GCONF_CLIENT_PRELOAD_ONELEVEL,
+        gconf_client_add_dir (client, KEY_DIR,
+                              GCONF_CLIENT_PRELOAD_ONELEVEL,
                               NULL);
-	gconf_client_notify_add (client,
+        gconf_client_notify_add (client,
                                  KEY_DIR,
-				 key_changed_cb,
+                                 key_changed_cb,
+                                 NULL, NULL, NULL);
+        gconf_client_add_dir (client, GNOME_LOCKDOWN_DIR,
+                              GCONF_CLIENT_PRELOAD_ONELEVEL,
+                              NULL);
+        gconf_client_notify_add (client,
+                                 GNOME_LOCKDOWN_DIR,
+                                 key_changed_cb,
                                  NULL, NULL, NULL);
 
         g_object_unref (client);
@@ -1599,11 +1598,7 @@ main (int    argc,
         textdomain (GETTEXT_PACKAGE);
 #endif
 
-        gnome_program_init (PACKAGE, VERSION,
-                            LIBGNOMEUI_MODULE,
-                            argc, argv,
-                            GNOME_PROGRAM_STANDARD_PROPERTIES,
-                            NULL);
+        gtk_init (&argc, &argv);
 
         job = gs_job_new ();
         theme_manager = gs_theme_manager_new ();
@@ -1615,5 +1610,5 @@ main (int    argc,
         g_object_unref (theme_manager);
         g_object_unref (job);
 
-	return 0;
+        return 0;
 }
